@@ -1,4 +1,3 @@
-import asyncio
 import importlib
 import os
 import platform
@@ -39,20 +38,23 @@ class FailureHost:
     error_messages: list[str]
 
 
+wrong_host_failure_host = FailureHost(
+    host="wrong.host.badssl.com",
+    error_messages=[
+        # OpenSSL
+        "Hostname mismatch, certificate is not valid for 'wrong.host.badssl.com'",
+        # macOS
+        "certificate name does not match",
+        # macOS with revocation checks
+        "certificates do not meet pinning requirements",
+        # macOS 10.13
+        "Recoverable trust failure occurred",
+        # Windows
+        "The certificate's CN name does not match the passed value.",
+    ],
+)
 failure_hosts_list = [
-    FailureHost(
-        host="wrong.host.badssl.com",
-        error_messages=[
-            # OpenSSL
-            "Hostname mismatch, certificate is not valid for 'wrong.host.badssl.com'",
-            # macOS
-            "certificate name does not match",
-            # macOS with revocation checks
-            "certificates do not meet pinning requirements",
-            # Windows
-            "The certificate's CN name does not match the passed value.",
-        ],
-    ),
+    wrong_host_failure_host,
     FailureHost(
         host="expired.badssl.com",
         error_messages=[
@@ -62,6 +64,8 @@ failure_hosts_list = [
             "“*.badssl.com” certificate is expired",
             # macOS with revocation checks
             "certificates do not meet pinning requirements",
+            # macOS 10.13
+            "Recoverable trust failure occurred",
             # Windows
             (
                 "A required certificate is not within its validity period when verifying "
@@ -79,6 +83,8 @@ failure_hosts_list = [
             "“*.badssl.com” certificate is not trusted",
             # macOS with revocation checks
             "certificates do not meet pinning requirements",
+            # macOS 10.13
+            "Recoverable trust failure occurred",
             # Windows
             (
                 "A certificate chain processed, but terminated in a root "
@@ -96,6 +102,8 @@ failure_hosts_list = [
             "“BadSSL Untrusted Root Certificate Authority” certificate is not trusted",
             # macOS with revocation checks
             "certificates do not meet pinning requirements",
+            # macOS 10.13
+            "Recoverable trust failure occurred",
             # Windows
             (
                 "A certificate chain processed, but terminated in a root "
@@ -112,6 +120,8 @@ failure_hosts_list = [
             "“superfish.badssl.com” certificate is not trusted",
             # macOS with revocation checks
             "certificates do not meet pinning requirements",
+            # macOS 10.13
+            "Recoverable trust failure occurred",
             # Windows
             (
                 "A certificate chain processed, but terminated in a root "
@@ -133,16 +143,11 @@ if platform.system() != "Linux":
             host="revoked.badssl.com",
             error_messages=[
                 # macOS
-                # "“revoked.badssl.com” certificate is revoked",
+                "“revoked.badssl.com” certificate is revoked",
+                # macOS 10.13
+                "Fatal trust failure occurred",
                 # Windows
-                # "The certificate is revoked.",
-                # TODO: Temporary while certificate is expired on badssl.com.
-                # Test will start failing against once the certificate is fixed.
-                # macOS
-                '"revoked.badssl.com","RapidSSL TLS DV RSA Mixed SHA256 2020 CA-1","DigiCert Global Root CA" certificates do not meet pinning requirements',
-                "“revoked.badssl.com” certificate is expired",
-                # Windows
-                "A required certificate is not within its validity period when verifying against the current system clock or the timestamp in the signed file.",
+                "The certificate is revoked.",
             ],
         )
     )
@@ -151,6 +156,24 @@ failure_hosts = decorator_requires_internet(
     pytest.mark.parametrize("failure", failure_hosts_list, ids=attrgetter("host"))
 )
 
+# Fixture which tests both the SecTrustEvaluate (macOS <=10.13) and
+# SecTrustEvaluateWithError (macOS >=10.14) APIs
+# on macOS versions that support both APIs.
+if platform.system() == "Darwin" and tuple(
+    map(int, platform.mac_ver()[0].split("."))
+) >= (10, 14):
+
+    @pytest.fixture(autouse=True, params=[True, False])
+    def mock_macos_version_10_13(request):
+        import truststore._macos
+
+        prev = truststore._macos._is_macos_version_10_14_or_later
+        truststore._macos._is_macos_version_10_14_or_later = request.param
+        try:
+            yield
+        finally:
+            truststore._macos._is_macos_version_10_14_or_later = prev
+
 
 @pytest.fixture
 def trustme_ca():
@@ -234,6 +257,8 @@ def test_failure_after_loading_additional_anchors(failure, trustme_ca):
         socket.create_connection((failure.host, 443)) as sock,
     ):
         ctx = truststore.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+        if platform.system() != "Linux":
+            ctx.verify_flags |= ssl.VERIFY_CRL_CHECK_CHAIN
 
         # See if loading additional anchors still fails.
         trustme_ca.configure_trust(ctx)
@@ -266,21 +291,25 @@ def test_sslcontext_api_success(host):
     ctx = truststore.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
     with urllib3.PoolManager(ssl_context=ctx, retries=5) as http:
         resp = http.request("GET", f"https://{host}")
-    assert resp.status == 200
+    # Cloudflare rejects some of our CI requests and
+    # that's okay because we only care about the TLS handshake.
+    assert resp.status in (200, 403)
     assert len(resp.data) > 0
 
 
 @successful_hosts
+@pytest.mark.filterwarnings("ignore:enable_cleanup_closed ignored.*:DeprecationWarning")
 @pytest.mark.asyncio
 async def test_sslcontext_api_success_async(host):
     ctx = truststore.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
-    async with aiohttp.ClientSession() as http:
+    # connector avoids https://github.com/aio-libs/aiohttp/issues/5426
+    async with aiohttp.ClientSession(
+        connector=aiohttp.TCPConnector(force_close=True, enable_cleanup_closed=True)
+    ) as http:
         resp = await http.request("GET", f"https://{host}", ssl=ctx)
 
         assert resp.status == 200
         assert len(await resp.text()) > 0
-    # workaround https://github.com/aio-libs/aiohttp/issues/5426
-    await asyncio.sleep(0.2)
 
 
 @failure_hosts
@@ -297,20 +326,21 @@ def test_sslcontext_api_failures(failure):
 
 
 @failure_hosts
+@pytest.mark.filterwarnings("ignore:enable_cleanup_closed ignored.*:DeprecationWarning")
 @pytest.mark.asyncio
 async def test_sslcontext_api_failures_async(failure):
     ctx = truststore.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
     if platform.system() != "Linux":
         ctx.verify_flags |= ssl.VERIFY_CRL_CHECK_CHAIN
-    async with aiohttp.ClientSession() as http:
+    # connector avoids https://github.com/aio-libs/aiohttp/issues/5426
+    async with aiohttp.ClientSession(
+        connector=aiohttp.TCPConnector(force_close=True, enable_cleanup_closed=True)
+    ) as http:
         with pytest.raises(
             aiohttp.client_exceptions.ClientConnectorCertificateError
         ) as e:
             await http.request("GET", f"https://{failure.host}", ssl=ctx)
 
-    # workaround https://github.com/aio-libs/aiohttp/issues/5426
-    await asyncio.sleep(0.2)
-
     assert "cert" in repr(e.value).lower() and "verif" in repr(e.value).lower()
 
 
@@ -346,6 +376,22 @@ def test_requests_sslcontext_api_failures(failure):
     assert "cert" in repr(e.value).lower() and "verif" in repr(e.value).lower()
 
 
+@pytest.mark.internet
+def test_wrong_host_succeeds_with_hostname_verification_disabled() -> None:
+    global wrong_host_failure_host
+
+    ctx = truststore.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+    ctx.check_hostname = False
+    assert ctx.check_hostname is False
+
+    with urllib3.PoolManager(ssl_context=ctx, retries=5, assert_hostname=False) as http:
+        resp = http.request("GET", f"https://{wrong_host_failure_host.host}")
+
+    assert resp.status == 200
+    assert len(resp.data) > 0
+    assert ctx.check_hostname is False
+
+
 def test_trustme_cert(trustme_ca, httpserver):
     ctx = truststore.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
     trustme_ca.configure_trust(ctx)
